@@ -7,7 +7,10 @@ import yahoo_fin.stock_info as si
 import pandas as pd
 import numpy as np
 import statsmodels.tsa.stattools as ts
-import statsmodels.api as sm
+from statsmodels.regression.rolling import RollingOLS
+
+import statsmodels.api as ssm
+import statsmodels.formula.api as smf
 
 from fredapi import Fred
 import pandas as pd
@@ -20,6 +23,9 @@ import pandas as pd
 from datetime import datetime
 from urllib.request import urlopen, Request
 
+import warnings
+
+
 # from nltk.sentiment.vader import SentimentIntensityAnalyzer
 
 web_url = "https://finviz.com/quote.ashx?t="
@@ -203,8 +209,6 @@ def get_price_data(ticker, reset=False, rolling_betas=True, rolling_window=60):
             ],
         )
 
-        from statsmodels.regression.rolling import RollingOLS
-
         model = RollingOLS.from_formula(
             "pct_change ~ mkt_change",
             data=df[["pct_change", "mkt_change"]],
@@ -233,7 +237,7 @@ def get_betas(x, y, n=0):
         y = y.iloc[
             -n:,
         ]
-    res = sm.OLS(y, x).fit()
+    res = ssm.OLS(y, x).fit()
     beta = res.params[0]
     r2 = res.rsquared
     n = len(x)
@@ -353,18 +357,22 @@ def get_earnings_dates(ticker):
     ]
 
 
-def regress_dataframe(df, y_col, x_col, y_filter):
+def regress_dataframe(df, y_col, x_col, y_filter, sort_col="", n=24):
     import datetime as ddt
-    import statsmodels.api as ssm
 
     if y_filter != "":
         df = df.loc[df[y_col] == y_filter]
 
-    X = df.head(24)[x_col]
-    Y = df.head(24)[y_col]
+    if sort_col != "":
+        df = df.sort_values(sort_col, ascending=False)
+
+    X = df.tail(n)[x_col]
+    Y = df.tail(n)[y_col]
 
     X = ssm.add_constant(X)  # to add constant value in the model
-    ols_model = ssm.OLS(Y.astype(float), X.astype(float)).fit()  # fitting the model
+    ols_model = ssm.OLS(
+        Y.astype(float), X.astype(float), missing="drop"
+    ).fit()  # fitting the model
     predictions = ols_model.summary()  # summary of the model
     return predictions
 
@@ -404,7 +412,7 @@ def get_earnings_and_prices(ticker):
             "mktclose_plus_10_return",
         ]
     ].sort_values("earnings_date", ascending=False)
-
+    df = df.groupby(["ticker", "earnings_date"]).first().reset_index()
     return df
 
 
@@ -412,9 +420,6 @@ def regress_dataframe_groups(
     df_data, y_name="alpha_10_day", return_grouped=True, n_periods=12
 ):
 
-    import statsmodels.api as ssm
-    import numpy as np
-
     # return df_data
     df_data = df_data[df_data[y_name].notna()]
     df_data = df_data[df_data["value"].notna()]
@@ -481,7 +486,6 @@ def plot_prices_against_time_series(
         right_on="earnings_date",
     )
     df_plot = df_plot.loc[df_plot["price_date"] > "2020-01-01"]
-    import matplotlib.pyplot as plt
 
     df_plot["row_num"] = np.arange(len(df_plot))
     markers_on = df_plot.dropna()
@@ -541,7 +545,6 @@ def plot_prices_against_time_series(
 
 
 def plot_eps_surprise_data(df):
-    import matplotlib.pyplot as plt
 
     df["z_score_alpha_10_day"] = (df["alpha_10_day"] - df["alpha_10_day"].mean()) / df[
         "alpha_10_day"
@@ -587,10 +590,29 @@ def plot_eps_surprise_data(df):
 
 
 def regress_dataframe_time_series_groups(
-    df_data=None, y_name="", return_grouped=True, n_periods=16
+    df_data=None,
+    y_name="",
+    return_grouped=True,
+    use_qoq_deltas=False,
+    use_yoy_deltas=False,
+    category_filter=[],
+    n_periods=16,
 ):
+
+    if len(category_filter):
+        df_data = df_data.loc[~df_data["category"].isin(category_filter)]
+
+    if use_qoq_deltas == False:
+        df_data = df_data.loc[
+            ~df_data["time_series_name"].str.contains("quarter_over_quarter")
+        ]
+
+    if use_yoy_deltas == False:
+        df_data = df_data.loc[
+            ~df_data["time_series_name"].str.contains("year_over_year")
+        ]
+
     ticker = df_data.iloc[0]["ticker"]
-    import canalyst_candas.candas_datareader as cdr
 
     df = regress_dataframe_groups(
         df_data, y_name=y_name, return_grouped=return_grouped, n_periods=n_periods
@@ -602,13 +624,14 @@ def regress_dataframe_time_series_groups(
     first_col = df.pop(col_name)
     df.insert(0, col_name, first_col)
 
+    df = df.loc[df["rsquared"] < 1.00]
+    df = df.loc[df["rsquared"] > 0]
     return df
 
 
 def regress_dataframe_groups(df_data, y_name="", return_grouped=True, n_periods=12):
 
-    import statsmodels.api as ssm
-    import numpy as np
+    warnings.filterwarnings("ignore", category=RuntimeWarning)
 
     # return df_data
     df_data = df_data.sort_values(["time_series_name", "period_end_date"])
@@ -617,28 +640,19 @@ def regress_dataframe_groups(df_data, y_name="", return_grouped=True, n_periods=
     # return df_data
 
     def extract_lr(x, y_name):
-        Y = x[y_name].astype(float).tail(n_periods)
-        y_len = Y.shape[0]
+        x = x.tail(n_periods)
+        model = smf.ols(formula="value ~ alpha_10_day", data=x)
+        rres = model.fit()
 
-        X = x[["value"]].tail(n_periods)
-
-        X = ssm.add_constant(X)
-        X = X.astype(float)
-        try:
-            model = ssm.OLS(Y, X).fit()
-            preds = model.predict(X)
-
-            x["preds"] = preds
-            x["rsquared"] = model.rsquared
-            x["slope"] = model.params[0]
-            x["intercept"] = model.params[1]
-
-            return pd.DataFrame(x)
-        except:
-            return None
+        x["intercept"] = rres.params[0]
+        x["n_periods"] = x.shape[0]
+        x["rsquared"] = rres.rsquared
+        x["slope"] = rres.params[1]
+        return pd.DataFrame(x)
 
     grouped = df_data.groupby("time_series_name")  # group by each time series name
     grouped = grouped.apply(lambda x: extract_lr(x, y_name))
+
     if return_grouped == True:
         df_output = (
             grouped.reset_index(drop=True)
@@ -655,6 +669,7 @@ def regress_dataframe_groups(df_data, y_name="", return_grouped=True, n_periods=
             .sort_values("rsquared", ascending=False)
         )
         df_output["model_row_number"] = df_output["name_index"]
+        df_output = df_output.loc[df_output["n_periods"] >= n_periods]
         return df_output[
             [
                 "model_row_number",
@@ -664,7 +679,46 @@ def regress_dataframe_groups(df_data, y_name="", return_grouped=True, n_periods=
                 "rsquared",
                 "slope",
                 "intercept",
+                "n_periods",
             ]
         ]
     else:
         return grouped
+
+
+def rolling_r_squared(df_data, time_series_name, window, plot=False):
+    category_filter = []
+    df = df_data[
+        df_data["time_series_name"] == time_series_name
+    ]  # [['period_name','value','earnings_date','alpha_10_day']]
+
+    model = RollingOLS.from_formula(
+        "value ~ alpha_10_day",
+        data=df[["value", "alpha_10_day"]],
+        window=window,
+    )
+    rres = model.fit()
+    df["intercept"] = rres.params[["Intercept"]]
+    df["slope"] = rres.params[["alpha_10_day"]]
+    df["rsquared"] = rres.rsquared
+    df = df[["alpha_10_day", "value", "slope", "rsquared", "intercept"]].dropna()
+    if plot == False:
+        return df
+
+    import matplotlib.pyplot as plt
+
+    fig, (ax1, ax2, ax3) = plt.subplots(nrows=3, sharex=True)
+
+    ax1.plot(df["slope"], label="Regression Slope")
+    ax1.legend(loc="upper left")
+
+    ax2.plot(df["rsquared"], label="R Squared")
+    ax2.legend(loc="upper left")
+
+    ax3.plot(df["value"], label="Time Series Value")
+    ax3.legend(loc="upper left")
+
+    ax1.get_shared_x_axes().join(ax1, ax2, ax3)
+    ax1.set_xticklabels([])
+    ax1.title.set_text(time_series_name)
+    plt.show()
