@@ -196,6 +196,7 @@ def filter_dataset(
     n_periods="",
     mrq_notation=False,
     unit_type="",
+    category="",
     warning=True,
 ):
 
@@ -216,6 +217,9 @@ def filter_dataset(
 
     df = df.sort_values(["period_end_date", "name_index"])
 
+    if category != "":
+        df = df.loc[df["category"] == category]
+
     if period_name != "":
         df = df.loc[df["period_name"] == period_name]
 
